@@ -61,10 +61,11 @@ c     integer ndummy
       character*128 title
       character*96 evid(nsites)
       save evid
-      real*8 cog, sumne,  obstimes, Savederg(5)
+      real*8 cog, cog2, sumne,  obstimes, Savederg(5)
+      real*8 depth, dd
       logical dosort
       character*2  kd(3)
-      save obstimes
+      save obstimes,  depth
 
 c       keep this interface so that the difference between
 c       the one in ForTA/  and in FleshHist is in the next line
@@ -389,17 +390,20 @@ c     *  eneded at this moment.
 c     *  After this is executed, event generation starts.
 c     *
       entry xBgEvent
-
-
       call cqIncident(inci, angle)
-      write(*,'("i ",  i3,  i4, g13.4,3f11.7)') EventNo, inci.p.code,
-     *   inci.p.fm.e,  -angle.r(1),  -angle.r(2), -angle.r(3)      
-c      
+
+       write(*,'("i ",  i3,  i4, g13.4,3f11.7,f7.1)')
+     *    EventNo, inci.p.code,
+     *   inci.p.fm.e,  -angle.r(1),  -angle.r(2), -angle.r(3)
+
+
+
       do i = 1, NoOfASSites
          SumEloss(i) = 0.
          Ng(i) = 0.
          Ne(i) = 0.
          Nmu(i) = 0.
+         Nhad(i) = 0.
       enddo
 
       if(tklat) then
@@ -523,6 +527,8 @@ c     For id =2, you need not output the z value, because it is always
 c     0 (within the computational accuracy).
 c
 c     **************************
+
+
       obstimes = obstimes + 1.d0
       if(mod(obstimes, 100000.d0) .eq. 0. ) then 
          dosort=.false.
@@ -559,8 +565,15 @@ c     ***************
             Ne(ldep) = Ne(ldep) + aTrack.wgt
          elseif(code .eq. kmuon) then
             Nmu(ldep) = Nmu(ldep) + aTrack.wgt
+         elseif(code .le. 6) then
+            Nhad(ldep) = Nhad(ldep) +aTrack.wgt
          endif
-
+c/////////////////
+c         if(aTrack.where .eq. 25 .and. 
+c     *      aTrack.p.code .eq. 1) then
+c            write(*,*) sngl(aTrack.wgt), sngl(aTrack.p.fm.p(4))
+c         endif
+c/////////////////
          do i = 1, ansites
             j = indivdep(i) 
             if( j .eq. ldep) then
@@ -619,7 +632,7 @@ c                     see if this is > K.E
                Eloss = Ek
                dist = Eloss/dedt
             endif
-            Eloss = Eloss*10.       !  GeV/(g/cm2)
+            Eloss = Eloss*aTrack.wgt       !  GeV/(g/cm2)
             SumEloss(ldep)=SumEloss(ldep) + Eloss
          endif
       endif
@@ -779,35 +792,86 @@ c                  re spectrum
 c     **************
       entry xEnEvent
 c     **************
+      call cqFirstID(depth)
+      depth = depth* 0.1     ! in g/cm2  First col depth.
 
       if(ObserveAS) then
          cog = 0.
          sumne = 0.
-         do i = 1, NoOfASSites
-            cog = cog + ASObsSites(i).esize*ASDepthList(i)*0.1
-            sumne= sumne +ASObsSites(i).esize
-         enddo
-         cog = cog/sumne
 
+         do i = 1, NoOfASSites
+            if(i .gt. 1 .and. i  .lt. NoOfASSites ) then
+               dd =(ASDepthList(i+1) - ASDepthList(i-1))/2.0
+            elseif(i .eq. 1) then
+               dd =(ASDepthList(2) - ASDepthList(1))
+            else
+               dd =(ASDepthList(NoOfASSites) -
+     *              ASDepthList(NoOfASSites-1))
+            endif
+            cog = cog + ASObsSites(i).esize*dd*ASDepthList(i)
+            sumne= sumne +ASObsSites(i).esize*dd
+         enddo
+c          0.1 is for g/cm2
+         cog = cog*0.1/sumne
+
+         cog2 = 0.
+         sumne = 0.
+         do i = 1, NoOfASSites
+            if( ASObsSites(i).age .gt.
+     *          (2.0-ASObsSites(NoOfASSites).age))  then
+               if(i .gt. 1 .and. i  .lt. NoOfASSites ) then
+                  dd =( ASDepthList(i+1) - ASDepthList(i-1))/2.0
+               elseif(i .eq. 1) then
+                  dd =(ASDepthList(2) - ASDepthList(1))
+               else
+                  dd =(ASDepthList(NoOfASSites) -
+     *              ASDepthList(NoOfASSites-1))
+               endif
+               dd = dd
+               cog2 = cog2 + ASObsSites(i).esize*ASDepthList(i)*dd
+               sumne= sumne +ASObsSites(i).esize*dd
+            endif
+         enddo
+         if(sumne .gt. 0.) then
+            cog2 = cog2*0.1/sumne
+         else
+c              too deep penetration
+            cog2 = ASDepthList(NoOfASSites)*0.1
+         endif
+
+
+         if(fnoB .ge. 0 )  then
+            write(fnoB,
+     *      '("h ", i4,  3i3, 1pE11.3, 0p 3f11.7, f7.2, 2f7.0)')
+     *      EventNo,  inci.p.code,
+     *      inci.p.subcode, inci.p.charge,
+     *      inci.p.fm.e, -angle.r(1), -angle.r(2), -angle.r(3),
+     *      depth, cog, cog2
+         else
+            write(*,
+     *      '("h ", i4,  3i3, 1pE11.3, 0p 3f11.7, f7.2, 2f7.0)')
+     *      EventNo,  inci.p.code,
+     *      inci.p.subcode, inci.p.charge,
+     *      inci.p.fm.e, -angle.r(1), -angle.r(2), -angle.r(3),
+     *      depth, cog, cog2
+         endif
          do i = 1, NoOfASSites 
             if(fnoB .ge. 0) then
-               write(fnoB, '( i4, i3, i5,
-     *          1pE11.3, 0pf5.2, f5.2, 1pE11.3, 0pf7.1,
-     *          1p3E11.3, 0pf7.0)')
-     *          EventNo, i,
-     *          int( ASDepthList(i)*0.1 ),  
-     *          sngl(ASObsSites(i).esize), sngl(ASObsSites(i).age),
-     *          ASDepthList(i)*0.1/cog, SumEloss(i),  
-     *          sngl(ASObsSites(i).mu), Ng(i), Ne(i), Nmu(i), cog
+               write(fnoB, '("t ", i3, 2f7.1,  2f6.3,
+     *         1p6E11.3)')
+     *          i, 
+     *          ASDepthList(i)*0.1,  ASObsSites(i).mu,
+     *          ASObsSites(i).age,   ASDepthList(i)*0.1/cog2, 
+     *          Ng(i), Ne(i), Nmu(i), Nhad(i),
+     *          ASObsSites(i).esize, SumEloss(i)  
             else
-               write(*, '( i4, i3, i5,
-     *          1pE11.3, 0pf5.2, f5.2, 1pE11.3, 0pf7.1,
-     *          1p3E11.3, 0pf7.0)')
-     *          EventNo, i,
-     *          int( ASDepthList(i)*0.1 ),  
-     *          sngl(ASObsSites(i).esize), sngl(ASObsSites(i).age),
-     *          ASDepthList(i)*0.1/cog, SumEloss(i),  
-     *          sngl(ASObsSites(i).mu), Ng(i), Ne(i), Nmu(i), cog
+               write(*, '("t ", i3, 2f7.1,  2f6.3,
+     *         1p6E11.3)')
+     *          i, 
+     *          ASDepthList(i)*0.1,  ASObsSites(i).mu,
+     *          ASObsSites(i).age,   ASDepthList(i)*0.1/cog2, 
+     *          Ng(i), Ne(i), Nmu(i), Nhad(i),
+     *          ASObsSites(i).esize, SumEloss(i)  
             endif
          enddo
       endif
@@ -823,8 +887,8 @@ c     **************
      *   '(i3, i5,  f5.2, f5.2,
      *   i5,  i4)')
      *   histdep(i), int( ASDepthList(j)*0.1 ),  
-     *   ASObsSites(j).age, ASDepthList(j)*0.1/cog,
-     *   int(ASObsSites(j).mu), int(cog)
+     *   ASObsSites(j).age, ASDepthList(j)*0.1/cog2,
+     *   int(ASObsSites(j).mu), int(cog2)
       enddo
 
 
