@@ -71,6 +71,8 @@ c       real*8 Zfirst       ! to keep first interaction slant depth
      *         )   ! max # of ptcls producable in coll.
 	record /ptcl/ Pwork(MaxPtcl)  ! working array to store ptcls.
 	integer Nproduced   ! no. of ptcls produced and stored in Pwork.
+        integer Nstacked    ! no. of ptcls stacked. If ThinSampling=F,
+                            ! same as Nproduced.  Nstacked <= Nproduced
 	real*8  MuonPolarization  ! muon polarization value.
 c
        common /Ztrackv/Pwork, IntInfArray, TrackBefMove,
@@ -79,7 +81,7 @@ c
      *  TargetMassN, TargetAtomicN, TargetZ2,
      *  EnergyLoss,  KEmin, KEminCas, Beta, 
      *  KEmin2, KEminCas2, Ethin, Upsilon, Xai, 
-     *  ObserveAS, Nproduced, 
+     *  ObserveAS, Nproduced, Nstacked,
      *  MoveStat,  NumberOfInte, ProcessNo, 
      *  TargetNucleonNo, TargetProtonNo, Upgoing, UseTbl,
      *  FromEpics
